﻿Imports PageLayoutControl.PrinterLayerDrawingUtilities
Imports ThinkGeo.Core
Imports ThinkGeo.UI.WinForms
Imports PageLayoutControl.DotNetUtils
Imports Microsoft.Win32
Imports System.Text
Imports System.IO

Public Class Form1
  Private msRoadsShapeFilePath As String = "..\..\Data\Roads.shp"
  Private msTestLineShapeFilePath As String = "..\..\Data\Test_Line_Shapefile.shp"
  Private msTestPointShapeFilePath As String = "..\..\Data\Test_Point_Shapefile.shp"
  Private msTestTractsNAD83UTM16NShapeFilePath As String = "..\..\Data\Tracts_NAD83UTM16N.shp"
  Private msTestTractsAlbersShapeFilePath As String = "..\..\Data\Tracts_Albers.shp"
  Private Const mcsLayers As String = "Layers"
  Private moLineShapefile As ShapeFileFeatureLayer
  Private moNad83_16N_TractShapefile As ShapeFileFeatureLayer
  Private moTracts_AlbersShapefile As ShapeFileFeatureLayer
  Private moPointsShapefile As ShapeFileFeatureLayer
  Private moAdornmentOverlay As AdornmentOverlay
  Private Async Sub Form1_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
    Me.WindowState = FormWindowState.Maximized
    PageLayout.CreateDefaultLayout()
    Dim oCurrentExtent As RectangleShape = PageLayout.PrinterPage.GetBoundingBox
    oCurrentExtent.ScaleUp(10)
    PageLayout.CurrentExtent = oCurrentExtent
    mLoadShapefiles()
    mPopulateProjectionCombo()
    moAdornmentOverlay = New AdornmentOverlay()
    'moAdornmentOverlay.
    'updatepapersize()
    Await PageLayout.RefreshAsync()
  End Sub
  Private Async Sub mLoadShapefiles()
    PageLayout.PageLayoutMap.MapUnit = GeographyUnit.Meter ' ThinkGeo.MapSuite.GeographyUnit.Meter
    Dim oProj4 As New ProjectionConverter
    With oProj4
      .InternalProjection = New Projection(DotNetUtils.EnumCoordSys.NAD83_UTM_zone_16N)
      .ExternalProjection = New Projection(DotNetUtils.EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic)
    End With
    oProj4.Open()

    moLineShapefile = New ShapeFileFeatureLayer(msTestLineShapeFilePath)
    moLineShapefile.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyle.CreateSimpleLineStyle(GeoColors.Blue, 2, True)
    moLineShapefile.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
    moLineShapefile.Open()

    PageLayout.PageLayoutMap.Layers.Add(moLineShapefile)

    moNad83_16N_TractShapefile = New ShapeFileFeatureLayer(msTestTractsNAD83UTM16NShapeFilePath)
    moNad83_16N_TractShapefile.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyle.CreateSimpleAreaStyle(GeoColors.Transparent, GeoColors.Red, 4)
    moNad83_16N_TractShapefile.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
    PageLayout.PageLayoutMap.Layers.Add(moNad83_16N_TractShapefile)

    moPointsShapefile = New ShapeFileFeatureLayer(msTestPointShapeFilePath)
    moPointsShapefile.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyle.CreateSimplePointStyle(PointSymbolType.Circle, GeoColors.Black, 12)
    moPointsShapefile.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
    PageLayout.PageLayoutMap.Layers.Add(moPointsShapefile)

    moTracts_AlbersShapefile = New ShapeFileFeatureLayer(msTestTractsAlbersShapeFilePath)
    Dim oStyle As New AreaStyle()
    Dim oImage As New GeoImage("..\..\Data\Swamp.png")
    oStyle.FillBrush = New GeoTextureBrush(oImage)

    'moTracts_AlbersShapefile.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColors.Transparent, GeoColors.Black, 2)
    moTracts_AlbersShapefile.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(oStyle)
    'moTracts_AlbersShapefile.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = oStyle
    moTracts_AlbersShapefile.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
    oProj4 = New ProjectionConverter
    With oProj4
      .InternalProjection = New Projection(DotNetUtils.EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic)
      .ExternalProjection = New Projection(DotNetUtils.EnumCoordSys.NAD83_UTM_zone_16N)
    End With
    oProj4.Open()
    moTracts_AlbersShapefile.FeatureSource.ProjectionConverter = oProj4
    PageLayout.PageLayoutMap.Layers.Add(moTracts_AlbersShapefile)

    moLineShapefile.Open()
    PageLayout.PageLayoutMap.Open()

    PageLayout.PageLayoutMap.MapExtent = MapUtil.GetDrawingExtent(moLineShapefile.GetBoundingBox, CSng(PageLayout.PageLayoutMap.GetBoundingBox().Width), CSng(PageLayout.PageLayoutMap.GetBoundingBox().Height))
    PageLayout.PageLayoutMap.SelectabilityMode = PrinterLayerSelectabilityMode.NonSelectable ' SelectabilityMode.NonSelectable
    PageLayout.ExtentOverlay.DoubleLeftClickMode = MapDoubleClickMode.Disabled ' ThinkGeo.MapSuite.WinForms.MapDoubleLeftClickMode.Disabled
    PageLayout.ExtentOverlay.DoubleRightClickMode = MapDoubleClickMode.Disabled ' ThinkGeo.MapSuite.WinForms.MapDoubleRightClickMode.Disabled
    PageLayout.ExtentOverlay.PanMode = MapPanMode.Disabled ' ThinkGeo.MapSuite.WinForms.MapPanMode.Disabled
    PageLayout.ExtentOverlay.MouseWheelMode = MapMouseWheelMode.Disabled ' ThinkGeo.MapSuite.WinForms.MapMouseWheelMode.Disabled

    Dim oOverlay As New LayerOverlay
    oOverlay.Layers.Add(moLineShapefile)
    oOverlay.Layers.Add(moNad83_16N_TractShapefile)
    oOverlay.Layers.Add(moPointsShapefile)
    oOverlay.Layers.Add(moTracts_AlbersShapefile)

    WinformsMap1.Overlays.Add(mcsLayers, oOverlay)
    WinformsMap1.MapUnit = GeographyUnit.Meter
    WinformsMap1.CurrentExtent = moLineShapefile.GetBoundingBox
    Await WinformsMap1.RefreshAsync()
  End Sub
  Private Sub Button1_Click(sender As Object, e As EventArgs)
    PageLayout.StartDrawing(EnumPrinterLayerDrawingType.Arrow)
  End Sub

  Private Sub btnDrawShape_Click(sender As Object, e As EventArgs)
    PageLayout.StartDrawing(EnumPrinterLayerDrawingType.Line)
  End Sub

  Private Sub btnDrawEllipse_Click(sender As Object, e As EventArgs)
    PageLayout.StartDrawing(EnumPrinterLayerDrawingType.Ellipse)
  End Sub
  Private Sub mUpdateProjection()
    Dim eSrid As EnumCoordSys = CType(cboProjection.Items(cboProjection.SelectedIndex), EnumCoordSys)
    'Private moLineShapefile As ShapeFileFeatureLayer
    'Private moNad83_16N_TractShapefile As ShapeFileFeatureLayer
    'Private moTracts_AlbersShapefile As ShapeFileFeatureLayer
    'Private moPointsShapefile As ShapeFileFeatureLayer
    Select Case eSrid
      Case EnumCoordSys.NAD83_UTM_zone_16N
        '***** The native coordinate system for many of the layers
        moLineShapefile.FeatureSource.ProjectionConverter = Nothing
        moNad83_16N_TractShapefile.FeatureSource.ProjectionConverter = Nothing
        moPointsShapefile.FeatureSource.ProjectionConverter = Nothing
        moTracts_AlbersShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic, EnumCoordSys.NAD83_UTM_zone_16N)
        WinformsMap1.MapUnit = GeographyUnit.Meter
      Case EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic
        moTracts_AlbersShapefile.FeatureSource.ProjectionConverter = Nothing
        moLineShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic)
        moPointsShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic)
        moNad83_16N_TractShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic)
        WinformsMap1.MapUnit = GeographyUnit.Meter
      Case EnumCoordSys.NAD83
        moLineShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.NAD83)
        moPointsShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.NAD83)
        moNad83_16N_TractShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.NAD83)
        moTracts_AlbersShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic, EnumCoordSys.NAD83)
        WinformsMap1.MapUnit = GeographyUnit.DecimalDegree
      Case EnumCoordSys.NAD_1983_StatePlane_Georgia_West_FIPS_1002_Feet
        moLineShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.NAD_1983_StatePlane_Georgia_West_FIPS_1002_Feet)
        moPointsShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.NAD_1983_StatePlane_Georgia_West_FIPS_1002_Feet)
        moNad83_16N_TractShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.NAD_1983_StatePlane_Georgia_West_FIPS_1002_Feet)
        moTracts_AlbersShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic, EnumCoordSys.NAD_1983_StatePlane_Georgia_West_FIPS_1002_Feet)
        WinformsMap1.MapUnit = GeographyUnit.Feet
      Case EnumCoordSys.Spherical_Mercator
        moLineShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.Spherical_Mercator)
        moPointsShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.Spherical_Mercator)
        moNad83_16N_TractShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.Spherical_Mercator)
        moTracts_AlbersShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic, EnumCoordSys.Spherical_Mercator)
        WinformsMap1.MapUnit = GeographyUnit.Meter
      Case EnumCoordSys.WGS_84
        moLineShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.WGS_84)
        moPointsShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.WGS_84)
        moNad83_16N_TractShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.NAD83_UTM_zone_16N, EnumCoordSys.WGS_84)
        moTracts_AlbersShapefile.FeatureSource.ProjectionConverter = New ProjectionConverter(EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic, EnumCoordSys.WGS_84)
        WinformsMap1.MapUnit = GeographyUnit.DecimalDegree
    End Select
    If moLineShapefile.FeatureSource.ProjectionConverter IsNot Nothing Then
      moLineShapefile.FeatureSource.ProjectionConverter.Open()
    End If
    If moNad83_16N_TractShapefile.FeatureSource.ProjectionConverter IsNot Nothing Then
      moNad83_16N_TractShapefile.FeatureSource.ProjectionConverter.Open()
    End If
    If moPointsShapefile.FeatureSource.ProjectionConverter IsNot Nothing Then
      moPointsShapefile.FeatureSource.ProjectionConverter.Open()
    End If
    If moTracts_AlbersShapefile.FeatureSource.ProjectionConverter IsNot Nothing Then
      moTracts_AlbersShapefile.FeatureSource.ProjectionConverter.Open()
    End If

    WinformsMap1.CurrentExtent = moPointsShapefile.GetBoundingBox
    PageLayout.PageLayoutMap.MapExtent = MapUtil.GetDrawingExtent(moPointsShapefile.GetBoundingBox, CSng(PageLayout.PageLayoutMap.GetBoundingBox().Width), CSng(PageLayout.PageLayoutMap.GetBoundingBox().Height))

    WinformsMap1.Refresh()
    PageLayout.Refresh()
  End Sub
  Private Sub mPopulateProjectionCombo()
    With cboProjection
      .Items.Add(EnumCoordSys.NAD83_UTM_zone_16N)
      .Items.Add(EnumCoordSys.NAD_1983_StatePlane_Georgia_West_FIPS_1002_Feet)
      .Items.Add(EnumCoordSys.Spherical_Mercator)
      .Items.Add(EnumCoordSys.USA_Contiguous_Albers_Equal_Area_Conic)
      .Items.Add(EnumCoordSys.NAD83)
      .Items.Add(EnumCoordSys.WGS_84)
      .SelectedIndex = 0
    End With

  End Sub
  Private Sub cboProjection_SelectionChangeCommitted(sender As Object, e As EventArgs) Handles cboProjection.SelectionChangeCommitted
    If cboProjection.SelectedIndex = -1 Then Return

    mUpdateProjection()

  End Sub
  Public Shared Function GetDriverConfigFilePath(ByVal driverName As String) As String()
    Try
      Dim driverKey As RegistryKey

      If Environment.Is64BitOperatingSystem Then
        driverKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\Wow6432Node\Microsoft\Windows NT\CurrentVersion\Print\Printers\" & driverName, False)
      Else
        driverKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\Microsoft\Windows NT\CurrentVersion\Print\Printers\" & driverName, False)
      End If

      Dim driverGUID As String = driverKey.GetValue("PrintQueueV4DriverDirectory").ToString()
      Dim driverConfigFileName As String = driverKey.OpenSubKey("PrinterDriverData").GetValue("V4_Merged_ConfigFile_Name").ToString()
      Dim driverConfig As String() = New String(1) {}
      driverConfig(0) = driverGUID
      driverConfig(1) = driverConfigFileName
      Return driverConfig
    Catch ex As Exception
      Console.WriteLine(ex)
      Return New String(1) {"-1", ""}
    End Try
  End Function

  Public Shared Function UpdateGPDFile(ByVal gpdPath As String) As Integer
      If File.Exists(gpdPath & ".backup") Then
        File.Delete(gpdPath & ".backup")
      End If

      File.Copy(gpdPath, gpdPath & ".backup")
      Console.WriteLine("Backuped file: " & gpdPath & ".backup")
      Dim sb As StringBuilder = New StringBuilder()
      sb.AppendLine("*Option: A6").AppendLine("{").AppendLine("*rcNameID: =RCID_DMPAPER_SYSTEM_NAME").AppendLine("*PrintSchemaKeywordMap: " & "ISOA6").AppendLine("*PrintableOrigin: PAIR(0, 0)").AppendLine("*PrintableArea: PAIR(1890000, 2664000)").Append("}")
      Dim pageSizeContent As String = sb.ToString()

      Try
        Return 0
      Catch ex As Exception
        Console.WriteLine(ex)
        Return -1
      End Try
    End Function

    Public Shared Function UpdateXMLFile(ByVal xmlPath As String) As Integer
      Return 0
    End Function

    Private Shared Sub UpdatePaperSize()
      'Console.WriteLine("Reading driver config file path from Registry for: Microsoft Print to PDF")
      Dim driverConfig As String() = GetDriverConfigFilePath("Microsoft Print to PDF")
      'Console.WriteLine("Read config file path from Registry: " & driverConfig(0) & "\" + driverConfig(1))
      Dim GPDresult As Integer = UpdateGPDFile("C:\Windows\System32\spool\V4Dirs\" & driverConfig(0) & "\" + driverConfig(1))
      Dim XMLresult As Integer = UpdateXMLFile("C:\Windows\System32\spool\V4Dirs\" & driverConfig(0) & "\" & "pdc.xml")
      'Console.ReadLine()
    End Sub
  End Class


